@@ -44,11 +44,13 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
+#ifndef G_OS_WIN32
+#include <pwd.h>
+#endif
 
 #include <glib/gi18n.h>
 #include <glib.h>
-
-#include <libgnome/gnome-util.h>
+#include <glib/gstdio.h>
 
 #include <libanjuta/anjuta-utils.h>
 #include <libanjuta/anjuta-debug.h>
@@ -459,7 +461,7 @@ anjuta_util_dialog_input (GtkWindow *parent, const gchar *prompt,
 }
 
 gboolean
-anjuta_util_prog_is_installed (gchar * prog, gboolean show)
+anjuta_util_prog_is_installed (const gchar * prog, gboolean show)
 {
 	gchar* prog_path = g_find_program_in_path (prog);
 	if (prog_path)
@@ -591,6 +593,35 @@ anjuta_util_glist_strings_dup (GList * list)
 	return new_list;
 }
 
+/* Join list of strings using the given delimiter */
+gchar*
+anjuta_util_glist_strings_join (GList * list, gchar *delimiter)
+{
+	GString *joined;
+	gboolean first = TRUE;
+	GList *node;
+	
+	joined = g_string_new (NULL);
+	node = list;
+	while (node)
+	{
+		if (node->data)
+		{
+			if (!first)
+				g_string_append (joined, delimiter);
+			else
+				first = FALSE;
+			g_string_append (joined, node->data);
+		}
+		node = g_list_next (node);
+	}
+	if (joined->len > 0)
+		return g_string_free (joined, FALSE);
+	else
+		g_string_free (joined, TRUE);
+	return NULL;
+}
+
 gchar*
 anjuta_util_get_real_path (const gchar *path)
 {
@@ -762,15 +793,105 @@ anjuta_util_update_string_list (GList *p_list, const gchar *p_str, gint length)
 }
 
 gboolean
-anjuta_util_create_dir (const gchar * d)
+anjuta_util_create_dir (const gchar* path)
 {
-	if (g_file_test (d, G_FILE_TEST_IS_DIR))
-		return TRUE;
-	if (mkdir (d, 0755))
+	GFile *dir = g_file_new_for_path (path);
+	GError *err = NULL;
+
+	if (g_file_query_exists (dir, NULL))
+	{
+		GFileInfo *info = g_file_query_info (dir, 
+				G_FILE_ATTRIBUTE_STANDARD_TYPE, 
+				G_FILE_QUERY_INFO_NONE, 
+				NULL, NULL);
+		if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
+		{
+			g_message ("Warning: %s is a file. \n \
+					It is trying to be treated as a directory.",g_file_get_path (dir));
+			g_object_unref (dir);
+			return FALSE;
+		}
+		g_object_unref (info);
+	}
+	else if (!g_file_make_directory (dir, NULL, &err))
+	{
+		g_warning ("Error directory:\n %s", err->message);
+		g_object_unref (dir);
 		return FALSE;
+	}
+	g_object_unref (dir);
+
 	return TRUE;
 }
 
+/**
+ * anjuta_util_user_shell:
+ * 
+ * Retrieves the user's preferred shell.
+ * 
+ * Returns: A newly allocated string that is the path to the shell.
+ */
+/* copied from deprecated gnome_util_user_shell in libgnome */
+char *
+anjuta_util_user_shell (void)
+{
+#ifndef G_OS_WIN32
+	struct passwd *pw;
+	int i;
+	const char *shell;
+	const char shells [][14] = {
+		/* Note that on some systems shells can also
+		 * be installed in /usr/bin */
+		"/bin/bash", "/usr/bin/bash",
+		"/bin/zsh", "/usr/bin/zsh",
+		"/bin/tcsh", "/usr/bin/tcsh",
+		"/bin/ksh", "/usr/bin/ksh",
+		"/bin/csh", "/bin/sh"
+											};
+
+	if (geteuid () == getuid () &&
+	    getegid () == getgid ()) {
+		/* only in non-setuid */
+		if ((shell = g_getenv ("SHELL"))){
+			if (access (shell, X_OK) == 0) {
+				return g_strdup (shell);
+			}
+		}
+	}
+	pw = getpwuid(getuid());
+	if (pw && pw->pw_shell) {
+		if (access (pw->pw_shell, X_OK) == 0) {
+			return g_strdup (pw->pw_shell);
+		}
+	}
+
+	for (i = 0; i != G_N_ELEMENTS (shells); i++) {
+		if (access (shells [i], X_OK) == 0) {
+			return g_strdup (shells[i]);
+		}
+	}
+
+	/* If /bin/sh doesn't exist, your system is truly broken.  */
+	abort ();
+
+	/* Placate compiler.  */
+											return NULL;
+#else
+	/* g_find_program_in_path() always looks also in the Windows
+ 	 * and System32 directories, so it should always find either cmd.exe
+ 	 * or command.com.
+ 	 */
+	char *retval = g_find_program_in_path ("cmd.exe");
+
+	if (retval == NULL)
+		retval = g_find_program_in_path ("command.com");
+
+	g_assert (retval != NULL);
+
+	return retval;
+#endif
+}
+
 pid_t
 anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 {
@@ -779,7 +900,7 @@ anjuta_util_execute_shell (const gchar *dir, const gchar *command)
 	
 	g_return_val_if_fail (command != NULL, -1);
 	
-	shell = gnome_util_user_shell ();
+	shell = anjuta_util_user_shell ();
 	pid = fork();
 	if (pid == 0)
 	{
@@ -896,6 +1017,7 @@ anjuta_util_parse_args_from_string (const gchar* string)
 	if (is_quote) {
 		g_warning ("Unclosed quotation encountered at the end of parsing");
 	}
+	g_free (buffer);
 	return args;
 }
 
@@ -926,6 +1048,7 @@ anjuta_util_escape_quotes(const gchar* str)
 	return buffer;
 }
 
+/* FIXME: Use gio instead */
 /* Diff the text contained in uri with text. Return true if files
 differ, FALSE if they are identical.
 FIXME: Find a better algorithm, this seems ineffective */
@@ -998,7 +1121,7 @@ anjuta_util_get_uri_mime_type (const gchar *uri)
 	 */
 	if (!path)
 	{
-		mime_type = gnome_vfs_get_mime_type (uri);
+		mime_type = gnome_vfs_get_slow_mime_type (uri);
 	}
 	else if (anjuta_util_path_has_extension (path, "anjuta"))
 	{
@@ -1018,7 +1141,7 @@ anjuta_util_get_uri_mime_type (const gchar *uri)
 	}
 	else
 	{
-		mime_type = gnome_vfs_get_mime_type (uri);
+		mime_type = gnome_vfs_get_slow_mime_type (uri);
 	}
 	
 	if (vfs_uri)
@@ -1319,3 +1442,339 @@ int scandir(const char *dir, struct dirent ***namelist,
 }
 
 #endif /* HAVE_LIBUTIL */
+
+void
+anjuta_util_help_display (GtkWindow   *parent,
+						  const gchar *doc_id,
+						  const gchar *file_name)
+{
+
+	GError *error = NULL;
+	GdkScreen *screen;
+	gchar *command;
+	const gchar *lang;
+	const gchar * const *langs;
+	gchar *uri = NULL;
+	gint i;
+	
+	g_return_if_fail (file_name != NULL);
+
+	langs = g_get_language_names ();
+	for (i = 0; langs[i]; i++)
+	{
+		lang = langs[i];
+		if (strchr (lang, '.'))
+			continue;
+
+		uri = g_build_filename (DATADIR, "/gnome/help/", doc_id,
+							    lang, file_name, NULL);
+
+		if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
+			break;
+		}
+		g_free (uri);
+		uri = NULL;
+	}
+
+	if (uri == NULL)
+	{
+		anjuta_util_dialog_error (parent, _("Unable to display help. Please make sure Anjuta "
+								  "documentation package is install. It can be downloaded "
+								  "from http://anjuta.org"));
+
+		return;
+	}
+	
+	command = g_strconcat ("gnome-help ghelp://", uri,  NULL);
+	g_free (uri);
+
+	screen = gtk_widget_get_screen (GTK_WIDGET (parent));
+	gdk_spawn_command_line_on_screen (screen, command, &error);
+	if (error != NULL)
+	{
+		g_warning ("Error executing help application: %s",
+				   error->message);
+		g_error_free (error);
+		
+		return;
+	}
+	g_free (command);
+}
+
+/* The following functions are taken from gedit */
+
+/* Note that this function replace home dir with ~ */
+gchar *
+anjuta_util_uri_get_dirname (const gchar *uri)
+{
+	gchar *res;
+	gchar *str;
+
+	// CHECK: does it work with uri chaining? - Paolo
+	str = g_path_get_dirname (uri);
+	g_return_val_if_fail (str != NULL, ".");
+
+	if ((strlen (str) == 1) && (*str == '.'))
+	{
+		g_free (str);
+		
+		return NULL;
+	}
+
+	res = anjuta_util_replace_home_dir_with_tilde (str);
+
+	g_free (str);
+	
+	return res;
+}
+
+gchar*
+anjuta_util_replace_home_dir_with_tilde (const gchar *uri)
+{
+	gchar *tmp;
+	gchar *home;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	/* Note that g_get_home_dir returns a const string */
+	tmp = (gchar *)g_get_home_dir ();
+
+	if (tmp == NULL)
+		return g_strdup (uri);
+
+	home = g_filename_to_utf8 (tmp, -1, NULL, NULL, NULL);
+	if (home == NULL)
+		return g_strdup (uri);
+
+	if (strcmp (uri, home) == 0)
+	{
+		g_free (home);
+		
+		return g_strdup ("~");
+	}
+
+	tmp = home;
+	home = g_strdup_printf ("%s/", tmp);
+	g_free (tmp);
+
+	if (g_str_has_prefix (uri, home))
+	{
+		gchar *res;
+
+		res = g_strdup_printf ("~/%s", uri + strlen (home));
+
+		g_free (home);
+		
+		return res;		
+	}
+
+	g_free (home);
+
+	return g_strdup (uri);
+}
+
+gchar *
+anjuta_util_str_middle_truncate (const gchar *string,
+								  guint        truncate_length)
+{
+	GString     *truncated;
+	guint        length;
+	guint        n_chars;
+	guint        num_left_chars;
+	guint        right_offset;
+	guint        delimiter_length;
+	const gchar *delimiter = "\342\200\246";
+
+	g_return_val_if_fail (string != NULL, NULL);
+
+	length = strlen (string);
+
+	g_return_val_if_fail (g_utf8_validate (string, length, NULL), NULL);
+
+	/* It doesnt make sense to truncate strings to less than
+	 * the size of the delimiter plus 2 characters (one on each
+	 * side)
+	 */
+	delimiter_length = g_utf8_strlen (delimiter, -1);
+	if (truncate_length < (delimiter_length + 2)) {
+		return g_strdup (string);
+	}
+
+	n_chars = g_utf8_strlen (string, length);
+
+	/* Make sure the string is not already small enough. */
+	if (n_chars <= truncate_length) {
+		return g_strdup (string);
+	}
+
+	/* Find the 'middle' where the truncation will occur. */
+	num_left_chars = (truncate_length - delimiter_length) / 2;
+	right_offset = n_chars - truncate_length + num_left_chars + delimiter_length;
+
+	truncated = g_string_new_len (string,
+				      g_utf8_offset_to_pointer (string, num_left_chars) - string);
+	g_string_append (truncated, delimiter);
+	g_string_append (truncated, g_utf8_offset_to_pointer (string, right_offset));
+		
+	return g_string_free (truncated, FALSE);
+}
+
+/* 
+ * Functions to implement XDG Base Directory Specification
+ * http://standards.freedesktop.org/basedir-spec/latest/index.html
+ * Use this to save any config/cache/data files
+ *
+ */
+
+static gchar*
+anjuta_util_construct_pathv (const gchar* str, va_list str_list) 
+{
+	GPtrArray *str_arr;
+	const gchar* tmp_str;
+	gchar* path;
+
+	str_arr = g_ptr_array_new();
+	g_ptr_array_add (str_arr, (gpointer) str);
+
+	/* Extract elements from va_list */
+	if (str != NULL)
+	{
+		while ((tmp_str = va_arg (str_list, const gchar*)) != NULL) 
+		{
+			g_ptr_array_add (str_arr, tmp_str);
+		}
+		va_end (str_list);
+	}
+	
+	/* Terminate the list */
+	g_ptr_array_add (str_arr, (gpointer) NULL);
+
+	path = g_build_filenamev ((gchar*) str_arr->pdata);
+	g_ptr_array_free (str_arr, TRUE);
+	
+	return path;
+}
+
+static GFile*
+anjuta_util_get_user_cache_filev (const gchar* path, va_list list)
+{
+	gchar *uri_str, *base_path, *dir;
+	GFile *uri;
+	const gchar anjuta_prefix[] = "anjuta";
+	base_path = g_build_filename (g_get_user_cache_dir(), anjuta_prefix, path, NULL);
+
+	uri_str = anjuta_util_construct_pathv (base_path, list);
+	g_free (base_path);
+
+	uri = g_file_new_for_path (uri_str);
+	dir = g_path_get_dirname (uri_str);
+	g_free(uri_str);
+	if (!anjuta_util_create_dir (dir)) return NULL;
+
+	return uri;
+}
+
+GFile*
+anjuta_util_get_user_cache_file (const gchar* path, ...)
+{
+	va_list list;
+	va_start (list, path);
+	return anjuta_util_get_user_cache_filev (path, list);
+}
+
+static GFile*
+anjuta_util_get_user_config_filev (const gchar* path, va_list list)
+{
+	gchar *uri_str, *base_path, *dir;
+	GFile *uri;
+	const gchar anjuta_prefix[] = "anjuta";
+	base_path = g_build_filename (g_get_user_config_dir(), anjuta_prefix, path, NULL);
+
+	uri_str = anjuta_util_construct_pathv (base_path, list);
+	g_free (base_path);
+
+	uri = g_file_new_for_path (uri_str);
+	dir = g_path_get_dirname (uri_str);
+	g_free(uri_str);
+	if (!anjuta_util_create_dir (dir)) return NULL;
+
+	return uri;
+}
+
+GFile*
+anjuta_util_get_user_config_file (const gchar* path, ...)
+{
+	va_list list;
+	va_start (list, path);
+	return anjuta_util_get_user_config_filev (path, list);
+}
+
+static GFile*
+anjuta_util_get_user_data_filev (const gchar* path, va_list list)
+{
+	gchar *uri_str, *base_path, *dir;
+	GFile *uri;
+	const gchar anjuta_prefix[] = "anjuta";
+	base_path = g_build_filename (g_get_user_data_dir(), anjuta_prefix, path, NULL);
+
+	uri_str = anjuta_util_construct_pathv (base_path, list);
+	g_free (base_path);
+
+	uri = g_file_new_for_path (uri_str);
+	dir = g_path_get_dirname (uri_str);
+	g_free(uri_str);
+	if (!anjuta_util_create_dir (dir)) return NULL;
+
+	return uri;
+}
+
+GFile*
+anjuta_util_get_user_data_file (const gchar* path, ...)
+{
+	va_list list;
+	va_start (list, path);
+	return anjuta_util_get_user_data_filev (path, list);
+}
+
+gchar*
+anjuta_util_get_user_cache_file_path (const gchar* path, ...)
+{
+	va_list list;
+	GFile *file;
+	gchar *file_path;
+	va_start (list, path);
+	file = anjuta_util_get_user_cache_filev (path, list);
+	file_path = g_file_get_path (file);
+	g_object_unref (file);
+
+	return file_path;
+}
+
+gchar*
+anjuta_util_get_user_config_file_path (const gchar* path, ...)
+{
+	va_list list;
+	GFile *file;
+	gchar *file_path; 
+	va_start (list, path);
+	file = anjuta_util_get_user_config_filev (path, list);
+	file_path = g_file_get_path (file);
+	g_object_unref (file);
+
+	return file_path;
+}
+
+gchar*
+anjuta_util_get_user_data_file_path (const gchar* path, ...)
+{
+	va_list list;
+	GFile *file;
+	gchar *file_path;;
+	va_start (list, path);
+	file = anjuta_util_get_user_data_filev (path, list);
+	file_path = g_file_get_path (file);
+	g_object_unref (file);
+
+	return file_path;
+}
+
